#!/usr/bin/perl

#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
use strict;
use DBI;

use libs::config;

my %conf = config::readHash("config/services.conf");

print "Connecting...\n";

my $dbh;
eval { $dbh = DBI->connect("DBI:mysql:".$conf{'mysql-db'}, $conf{'mysql-user'}, $conf{'mysql-pass'}, {  AutoCommit => 1, RaiseError => 1 }) };

if($@) {
	print "FATAL: Can't connect to database:\n$@\n";
	print "You must edit config/services.conf and create a corresponding\nMySQL user and database!\n\n";
	exit;
}

print "Creating tables...\n";

$dbh->{RaiseError} = 0;
$dbh->{PrintError} = 0;

open SQL, "services.sql";
my $sql;
while(my $x = <SQL>) { $sql .= $x unless $x =~ /^#/ or $x eq '\n'}
foreach my $line (split(/;/s, $sql)) {
	$dbh->do($line);
}

my $add_perm = $dbh->prepare("INSERT IGNORE INTO chanperm SET name=?, level=?");
my $del_perm = $dbh->prepare("DELETE FROM chanperm WHERE name=?");

my @perms = (
        ['Join', 0],
        ['AccList', 1],
        ['AccChange', 5],
        ['AKICK', 5],
	['AKickList', 3],
        ['SET', 6],
        ['BAN', 4],
        ['CLEAR', 6],
        ['GETKEY', 4],
        ['INFO', 0],
        ['KICK', 4],
	['LEVELS', 6],
	['LevelsList', 3],
        ['INVITE', 4],
        ['TOPIC', 5],
        ['UNBAN', 4],
        ['VOICE', 2],
        ['HALFOP', 3],
        ['OP', 4],
        ['PROTECT', 5],
        ['Memo', 5],
        ['BadWords', 5],
        ['Greet', 1],
        ['NoKick', 4],
        ['BotSay', 5],
        ['BotAssign', 6],
        ['SetTopic', 0],
	['WELCOME', 6]
);

my @noperms = ();

foreach my $p (@perms) {
	$add_perm->execute($p->[0], $p->[1]);
}

foreach my $p (@noperms) {
	$del_perm->execute($p);
}

print "Database setup complete!\n";
