#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package modes;

use strict;
no strict 'refs';
use constant {
	DEBUG => 0,

	DIFF => 1,
	ADD => 2,
	MERGE => 3
};

# Skip channel modes with arguments:
my $scm = qr/^[bevhoaq]$/;

# Allowed channel modes:
my $acm = qr/^[cfiklmnprstzACGMKLNOQRSTVu]$/;

# This gives what we need to do to bring a current modeset into compliance
# with a specified modeset (used for modelock)
sub diff($$$) {
	return calc($_[0], $_[1], $_[2], DIFF);
}

# This gives the result of applying the mode changes in the second parameter
# to the existing modes in the first parameter.
sub add($$$) {
	return calc($_[0], $_[1], $_[2], ADD);
}

# This gives back the modes in the first parameter, with any modes in the
# second overriding the first. (used to validate modelock setting)
sub merge($$$) {
	return calc($_[0], $_[1], $_[2], MERGE);
}

sub invert($) {
	my @modes = split(/ /, $_[0]);

	$modes[0] =~ tr/+-/-+/;

	return join(' ', @modes);
}

#  This is far from the best way to do it.
#  
#  'bekfLlvhoaq' 'kfLl'
#  
sub calc($$$$) {
	my ($src, $dst, $chan, $type) = @_;

	my ($smodes, @sargs) = split(/ /, $src);
	my ($dmodes, @dargs) = split(/ /, $dst);

	#$smodes =~ s/[bevhoaq]//g if $chan;
	
	my @smodes = split(//, $smodes);
	my @dmodes = split(//, $dmodes);
	
	my $sign = 2;
	my (@tmodes, @targs, @omodes, @oargs, $rmodes, @rargs);
	
	foreach my $x (@smodes) {
		if($x eq '+') { $sign=2; next; }
		if($x eq '-') { $sign=1; next; }
		if($chan and $x =~ $scm) {
			shift @sargs if($sign == 2);
			next;
		}
		if($chan and $x !~ $acm) {
			next;
		}
	
		if($type == DIFF or $type == ADD) {
			$tmodes[ord($x)] = $sign if $type == DIFF;
			$omodes[ord($x)] = $sign if $type == ADD;
			
			if($chan and $sign == 2 and $x =~ /^[kfLl]$/) {
				$targs[ord($x)] = shift @sargs if $type == DIFF;
				$oargs[ord($x)] = shift @sargs if $type == ADD;
			}
		}
		
		elsif($type == MERGE) {
			if($chan and $sign == 2 and $x =~ /^[kfLl]$/) {
				if(
					($x eq 'l' and $sargs[0] =~ /^\d+$/) or
					($x eq 'L' and $sargs[0] =~ /^#/) or
					$x eq 'f' or $x eq 'k'
				) {
					$omodes[ord($x)] = $sign;
					$oargs[ord($x)] = shift @sargs;
				}
			} else {
				$omodes[ord($x)] = $sign;
			}
		}
	}

	foreach my $x (@dmodes) {
		if($x eq '+') { $sign=2; next; }
		if($x eq '-') { $sign=1; next; }
		if($chan and $x =~ $scm) {
			shift @dargs if($sign == 2);
			next;
		}
		if($chan and $x !~ $acm) {
			next;
		}

		if($chan and $sign == 2 and $x =~ /^[kfLl]$/) {
			$oargs[ord($x)] = shift @dargs;
		}

		if(
			$type == ADD or
			$type == MERGE or
			$type == DIFF and (
				($sign==2 or $tmodes[ord($x)]) and (
					$sign != $tmodes[ord($x)] or
					$targs[ord($x)] ne $oargs[ord($x)]
				)
			)
		) {
			$omodes[ord($x)] = $sign;
		}

		# -k won't work without its parameter!
		if($chan and $type == DIFF and $sign == 1 and $x eq 'k') {
			$oargs[ord($x)] = $targs[ord($x)];
		}
	}

	$sign = 0;
	for(my $i; $i<@omodes; $i++) {
		if($omodes[$i] == 2) {
			if($sign != 2) { $sign = 2; $rmodes .= '+'; }
			$rmodes .= chr($i);
			push @rargs, $oargs[$i] if $oargs[$i];
			
		}
	}
	
	if($type == DIFF or $type == MERGE) {
		for(my $i; $i<@omodes; $i++) {
			if($omodes[$i] == 1) {
				if($sign != 1) { $sign = 1; $rmodes .= '-'; }
				$rmodes .= chr($i);
				push @rargs, $oargs[$i] if $oargs[$i];
			}
		}
	}

	#return undef if($rmodes eq '+');
	print "modes::calc($src, $dst, $chan, $type)\n" if DEBUG();
	print "--- MODE CALCULATED: ", join(' ', $rmodes, @rargs), "\n" if DEBUG();
	return join(' ', $rmodes, @rargs);
}

1;
