#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package thread;

use strict;
no strict 'refs';
use threads;
use threads::shared;

my $debug = 0;

our $shutdown : shared = 0;

our @inqueue : shared;
our @outqueue : shared;
our $timercond : shared;

sub shutdown() {
	net::agent_quit_all("Shutting down.");
	sleep 1;
	
	$shutdown = 1;
		
	net::ping();

	{
		lock @thread::inqueue;
		cond_broadcast(@thread::inqueue);
	}

	{
		lock @thread::outqueue;
		cond_broadcast(@thread::outqueue);
	}

	{
		lock $thread::timercond;
		cond_broadcast($thread::timercond);
	}
}

sub ircrecv($) {
	{
		lock @inqueue;
		push @inqueue, $_[0];
		cond_signal(@inqueue);
	}
}

sub ircsend(@) {
	print "Thread ".threads->self->tid()." generated output.\n" if $debug;
	my $out = join("\r\n", @_);

	{
		lock @outqueue;
		push @outqueue, $out;
		cond_signal(@outqueue);
	}
}

sub worker() {
	main::beginmod();

	INQUEUE: while(!$shutdown) {
		my $line;
		while(!defined($line)) {
			lock @inqueue;
			
			unless(@inqueue) {
				cond_wait(@inqueue);
				last INQUEUE if($shutdown);
			}
			
			$line = shift @inqueue;
		}
		
		print "Thread ".threads->self->tid()." got input.\n" if $debug;
		net::dispatch($line);
	}

	print "Worker thread dying!\n";

	main::endmod();
}

sub writer() {
	OUTQUEUE: while(1) {
		my $line;
		
		while(!defined($line)) {
			lock @outqueue;
	
			unless(@outqueue) {
				last OUTQUEUE if $shutdown;
				cond_wait(@outqueue);
				last OUTQUEUE if $shutdown;
			}
	
			$line = shift @outqueue;
		}
	
		net::ircsendreal($line);
	}

	$shutdown = 2;
}

1;
