#!/usr/bin/perl

#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use threads;
use threads::shared;

use Cwd 'abs_path';
use File::Basename;

use libs::config;
use libs::thread;
use libs::event;
use libs::net;
use libs::help;
use libs::modes;
use libs::log;
use libs::email;
use libs::fmt;
use libs::misc;

use IO::Handle;

our $prefix = dirname(abs_path($0));
*notice = \&nickserv::notice;

config::loadconfig();

STDOUT->autoflush(1);

our $debug = 1;
our $ssnick = 'ServServ2';
our $progname = 'SurrealServices';
our $version = '0.3.3';
our $extraversion = 'configured for UnrealIRCd 3.2.2 and up';

foreach $module (split(/\s*,\s*/, $config::conf{load})) {
	my $m = "./modules/$module.pm";
	print "Loading module $module...";
	require $m;
	$m = "$module\::init";
	eval { &$m };
	print " done.\n";
}

print "Initializing...";

my @workers; my $writer; my $timer;

net::agent_connect($ssnick, 'service', undef, '+opqBHS', 'Services Control Agent');

event::addhandler('EOS', undef, undef, __PACKAGE__, 'main::ev_connect');

sub ev_connect {
	#net::agent_connect($ssnick, 'service', undef, '+opqBHS', 'Services Control Agent');
	net::join($ssnick, $config::conf{diag});
	net::setmode($ssnick, $config::conf{diag}, '+o', $ssnick);
	event::addtimer('perlserv__pingtimer', 60, __PACKAGE__,
			"main::pingtimer");
}

event::addhandler('PRIVMSG', undef, 'servserv', __PACKAGE__, 'main::dispatch');

sub dispatch {
#	my ($src, $dst, $msg) = @_;
	my $user = { NICK => $_[0], AGENT => $ssnick };
	if(!adminserv::is_ircop($user)) {
		notice($user, 'Access Denied');
		net::globops($ssnick, "\002$src\002 failed access to ServServ $msg");
		return;
	}
	if($_[2] =~ /^lsmod/i) {
		notice($user, $config::conf{load});
	}

	if($_[2] =~ /^lsjoin/i) {
		foreach my $agent (keys(%net::agents)) {
			notice($user, "$agent: " . join(', ', keys(%{$net::agents{lc $agent}{CHANS}})));
		}
	}

	if($_[2] =~ /^shutdown/i) {
		if(!adminserv::is_svsop($user, adminserv::S_ADMIN() )) {
			notice($user, 'You do not have sufficient rank for this command');
			return;
		}
		
		thread::shutdown();
	}
	if($_[2] =~ /^raw/i) {
	    if(!adminserv::is_svsop($user, adminserv::S_ROOT() )) {
			notice($user, 'You do not have sufficient rank for this command');
			return;
		}
		my $cmd = $_[2];
		$cmd =~ s/raw\s+//;
		net::ircsend($cmd);
	}
}

#net::serv_connect();

sub pingtimer($) {
	net::ping();
	event::addtimer('perlserv__pingtimer', 60, __PACKAGE__, 
			"main::pingtimer");
}

sub beginmod() {
	foreach my $module (split(/,\s+/, $config::conf{load})) {
		my $m = "$module\::begin";
		eval { &$m };
		print $@ if $@;
	}
}

sub endmod() {
	foreach my $module (split(/,\s+/, $config::conf{load})) {
		my $m = "$module\::end";
		eval { &$m };
		print $@ if $@;
	}
}

print " done.\nConnecting...";

net::serv_connect();
print " done.\n";

unless($debug) {
	exit if fork;
	close STDIN;
	close STDOUT;
}

$writer = threads->new(\&thread::writer);
for(my $i=0; $i < $config::conf{procs}; $i++) {
	push @threads, threads->new(\&thread::worker);
}

$timer = threads->new(\&event::checktimers);
#event::checktimers();
#net::serv_connect();

#print " done.\n\n";

net::ircrecv();

$writer->join;
foreach my $t (@threads) { $t->join }
$timer->join;

foreach my $module (split(/,\s+/, $config::conf{load})) {
	my $m = "$module\::unload";
	if(defined(&$m)) {
		eval { &$m };
		print $@ if $@;
	}
}

print "Main thread dying!\n" if $debug;
